# from utils.secretStore import SecretStore
# user = SecretStore()
# decrypted_text = user.decrypt()
# print(decrypted_text)
#
# from utils.helperFunctions import SentimentAnalysis
#
# sa = SentimentAnalysis()
#
# res_ = sa.get_sentiment("SoftBank Group Corp. sold a majority of its stake in Paytm before regulatory scrutiny caused the once-celebrated Indian fintech firm’s shares to dive, according to the Vision Fund’s executive managing partner, as per a Bloomberg report.The Tokyo-based tech investor saw uncertainty growing in India’s regulatory environment, as well as over Paytm Payments Bank Ltd.’s license, Navneet Govil told Bloomberg News on February 8.“We felt it was prudent to start monetizing, the Vision Fund’s finance chief said, as quoted by Bloomberg.Were glad we did a good portion of Paytm before the recent stock correction.According to a Bloomberg analysis of company filings, SoftBank has been consistently selling off Paytm shares since at least November 2022 until last month. The Japanese investors stake in Paytm stood at approximately 5% as of January, a significant decrease from the approximately 18.5% stake it held around the time of Paytms initial public offering in 2021.When asked about SoftBanks plans regarding its remaining stake, Govil declined to comment.Earlier Paytm has received multiple warnings from regulators over the past two years regarding transactions between its popular payments app and its banking arm. Consequently, much of the banking operations business has been suspended by the Reserve Bank of India, leading to a decline of over 40% in Paytms stock price from its peak in January, as reported by Bloomberg.SoftBank earlier reported its first profit following four quarters of losses, with its Vision Fund also logging a profit in the December quarter. New bets by the startup investment arm have dwindled to a fraction of the billions it once wielded and have been outpaced by exit activity, however.Milestone Alert! Livemint tops charts as the fastest growing news website in the world 🌏 Click here to know more.Here’s your comprehensive 3-minute summary of all the things Finance Minister Nirmala Sitharaman said in her Budget speech: Click to download!")
# print(res_[0])


from utils.webscraping import GoogleNews
from utils.helpers import convertToPdf

gnews = GoogleNews()
res = gnews.get_news(sentiment=True, query="Paypal", max_results=50)

convertToPdf(res, "CP")


# import pickle
# file = open('BLTD.txt', 'wb')
# pickle.dump(res, file)
# file.close()

# for i in res:
#     print(i)

# from nlpretext import Preprocessor
# text = "nkmk;. m,k nd 😀 #food #paris \n </div>"
# preprocessor = Preprocessor()
# text = preprocessor.run(text)
# print(text)


# import pickle
#
# # open a file, where you stored the pickled data
# file = open('res.txt', 'rb')
#
# # dump information to that file
# data = pickle.load(file)
#
# # close the file
# file.close()

# from utils.helpers import SentimentAnalysis
#
# sa = SentimentAnalysis()
# sa.get_sentiment(data[0]["Text"].decode('utf-8'))
#
# print(sa.label)




import json
import random
import time
import email.utils
import requests
import iso639
from langdetect import detect
from bs4 import BeautifulSoup
# from fastapi.security import http
from gnews import GNews
from trustplutusmr.utils.logs import loga
import trustplutusmr.config.constants as ct
import lxml
from newspaper import Article, Config
from datetime import datetime
from nlpretext import Preprocessor
from trustplutusmr.utils.helpers import SentimentAnalysis, clean_text, TextChunk
from pydantic import BaseModel, Field
# from utils.secretStore import SecretStore
from trafilatura import fetch_url, extract

class GoogleNews(BaseModel):
    # Max number is 100. Getting more than 100 results increases chance of getting blocked
    max_results: int = Field(100, lt=101)

    # Format for period field
    # hours (eg: 1h), day (eg:2d), months (eg: 1m), years (eg:1y)
    period: str = None

    # Format for start and end date : Tuple(dd,mm,yyyy)
    end_date: datetime = None
    start_date: datetime = None

    language: str = "en"
    country: str = "IN"

    # store = SecretStore()
    # secrets: dict = store.decrypt(secret_keys=["ScrapperAPI"])

    @staticmethod
    def get_article(link):
        article = Article(link)
        article.download()
        article.parse()
        article.nlp()
        return article

    @staticmethod
    def extract_article_html(url):
        config = Config()
        config.fetch_images = True
        config.request_timeout = 30
        config.keep_article_html = True
        article = Article(url, config=config)

        article.download()
        article.parse()

        article_html = article.article_html

        html = lxml.html.fromstring(article_html)
        for tag in html.xpath('//*[@class]'):
            tag.attrib.pop('class')

        return lxml.html.tostring(html).decode('utf-8')

    # @loga
    def get_news(self, query, sentiment: bool = False, start_date: tuple = None,
                 end_date: tuple = None, country: str = None, period: str = None,
                 max_results: int = None):

        google_news = GNews()
        google_news.language = self.language
        google_news.period = self.period if period is None else period
        google_news.max_results = self.max_results if max_results is None else max_results
        google_news.country = self.country if country is None else country
        google_news.start_date = self.start_date if start_date is None else start_date
        google_news.end_date = self.end_date if end_date is None else end_date

        query_news = google_news.get_news(query)
        print("Finished scraping Google News")
        import pickle
        file = open('dump.txt', 'wb')
        pickle.dump(query_news, file)
        file.close()
        print("Processing articles...")
        processed_feed = []
        preprocessor = Preprocessor()
        for i in query_news:
            article_dict = {"title": i["title"], "date": i["published date"]}
            try:
                article = self.get_article(i["url"])
                article_dict["url"] = url = dict(article.meta_data)["og"]["url"]
                article_dict["summary"] = preprocessor.run(article.summary)
                document = fetch_url(url)
                text = extract(document)
                article_dict["text"] = preprocessor.run(text)
                processed_feed.append(article_dict)
            except Exception as e:
                print("Newspaper3k error: {0}, skipping article.".format(e.__class__.__name__))
                print("Exception details: ", e)
        print("Done")

        feed_with_sentiment = []
        if sentiment:
            print("Analyzing Sentiment...")
            for i in processed_feed:
                text = clean_text(i["title"])
                result = detect(text)
                lang_name = iso639.Language.from_part1(result).name
                if lang_name == 'English':
                    sa = SentimentAnalysis()
                    sa.get_sentiment(text)
                    i["sentiment"] = sa.label
                    print("Overall Sentiment: ", i["sentiment"])
                    feed_with_sentiment.append(i)
                    article_html = ''
                    try:
                        article_html = self.extract_article_html(i["url"])
                    except Exception as e:
                                print("Newspaper3k error: {0}, skipping article.".format(e.__class__.__name__))
                                print("Exception details: ", e)
                    i["articleHtml"] = '<div> <center> <h2>'+ i["title"] +' </h2> <center> <h4>Date: '+ i["date"] +'</h4><h4>Sentiment: '+str(i["sentiment"])+'</h4> </div><a></a><hr>' + article_html
                else:
                    continue
                # time.sleep(random.uniform(1.5, 5.5))
            print("Done")
            return feed_with_sentiment
        return processed_feed


    # def get_gserp(self,query):
    #     # Scarper API
    #     payload = {
    #         'api_key': self.secrets["SrcrapperAPI"],
    #         'country': 'in',
    #         'query': query
    #     }
    #     response = requests.get(
    #         'https://api.scraperapi.com/structured/google/search', params=payload)
    #     response_dict = json.loads(response.text)
    #
    #     result = []
    #     for i in response_dict["organic_results"]:
    #         print("Snippet: ",i["snippet"])
    #         page = http.request("GET",i["link"])
    #         print("Link: ",i["link"])
    #         soup = BeautifulSoup(page.data)
    #         try:
    #             text = soup.body.get_text(strip=True)
    #         except:
    #             pass
    #         else:
    #             print("-----------------")



import json
import pytz
import spacy
import statistics
import email.utils
import numpy as np
import pandas as pd
from xhtml2pdf import pisa
from trustplutusmr.utils.logs import loga
import trustplutusmr.config.constants as ct
from pydantic import BaseModel
from tzlocal import get_localzone
from transformers import pipeline
from nlpretext import Preprocessor
from trustplutusmr.config.constants import json_path
# from utils.secretStore import SecretStore
from transformers import AutoTokenizer, AutoModelForSequenceClassification
from xhtml2pdf import pisa

def convert_html_to_pdf(html_string, pdf_path):
    with open(pdf_path, "wb") as pdf_file:
        pisa_status = pisa.CreatePDF(html_string, dest=pdf_file)
    return not pisa_status.err

class Var(BaseModel):

    @staticmethod
    def read(keys: list):
        f = open(json_path)
        json_ = json.load(f)
        result = [json_.get(key) for key in keys]
        f.close()
        return result

    @staticmethod
    def add(key_value: dict):
        with open(json_path, 'w+') as f:
            json.dump(key_value, indent=4, sort_keys=True, fp=f)


class SentimentAnalysis(BaseModel):
    sent_models: list[str] = [ct.sent_model_1, ct.sent_model_2, ct.sent_model_3]
    labels: list[float] = []
    label: int = 0
    # @loga
    def get_sentiment(self, text):
        for i in self.sent_models:
            tokenizer = AutoTokenizer.from_pretrained(i)
            model = AutoModelForSequenceClassification.from_pretrained(i)
            nlp = pipeline("sentiment-analysis", model=model, tokenizer=tokenizer)
            results = nlp(text)
            self.labels.append(results[0]["label"].lower())

        mv_label = statistics.mode(self.labels)
        if mv_label == "positive":
            self.label = 1
        elif mv_label == "negative":
            self.label = 0
        elif mv_label == "neutral":
            self.label = 0.5


# Text cleaner
def clean_text(text):
    preprocessor = Preprocessor()
    return preprocessor.run(text)


class TextChunk(BaseModel):
    # Initialize the clusters lengths list and final texts list
    clusters_lens: list = []
    final_texts: list = []

    @staticmethod
    # Divide text into sentences and generate vector embeddings
    def process(text):

        # Load the Spacy model -> python -m spacy download en_core_web_lg
        nlp = spacy.load('en_core_web_lg')
        doc = nlp(text)
        sents = list(doc.sents)
        vecs = np.stack([sent.vector / sent.vector_norm for sent in sents])
        return sents, vecs

    @staticmethod
    def cluster_text(sents, vecs, threshold):
        clusters = [[0]]
        for i in range(1, len(sents)):
            if np.dot(vecs[i], vecs[i - 1]) < threshold:
                clusters.append([])
            clusters[-1].append(i)
        return clusters

    def get_chunks(self, data):

        # Process the chunk
        sents, vecs = self.process(data)

        # Cluster the sentences
        clusters = self.cluster_text(sents=sents, vecs=vecs, threshold=0.3)

        for cluster in clusters:
            cluster_txt = clean_text(' '.join([sents[i].text for i in cluster]))
            cluster_len = len(cluster_txt)

            # Check if the cluster is too short
            if cluster_len < 60:
                continue

            # Check if the cluster is too long
            elif cluster_len > 1000:
                sents_div, vecs_div = self.process(cluster_txt)
                reclusters = self.cluster_text(sents=sents_div, vecs=vecs_div, threshold=0.6)

                for subcluster in reclusters:
                    div_txt = clean_text(' '.join([sents_div[i].text for i in subcluster]))
                    print(div_txt)
                    print("----------")
                    div_len = len(div_txt)

                    if div_len < 60 or div_len > 3000:
                        continue

                    self.clusters_lens.append(div_len)
                    self.final_texts.append(div_txt)

            else:
                self.clusters_lens.append(cluster_len)
                self.final_texts.append(cluster_txt)


# class GenerateGraph(BaseModel):
#     path = ct.graph_path
#
#     @staticmethod
#     def utc_to_local(data):
#         utc_dt = email.utils.parsedate_to_datetime(data["date"])
#         local_tz = get_localzone()
#         local_dt = utc_dt.replace(tzinfo=pytz.utc).astimezone(local_tz)
#         data["date_parsed"] = local_dt
#         return data
                
# Utility function
def convert_html_to_pdf(source_html, output_filename):
    # open output file for writing (truncated binary)
    result_file = open(output_filename, "w+b")

    # convert HTML to PDF
    pisa_status = pisa.CreatePDF(
            source_html,                # the HTML to convert
            dest=result_file)           # file handle to recieve result

    # close output file
    result_file.close()                 # close output file

    # return False on success and True on errors
    return pisa_status.err

def convertToPdf(data, outFile):
    
    # file = open(fileName+'.txt', 'rb')
    # data = pickle.load(file)
    # file.close()
    
    result = pd.DataFrame.from_dict(data)
    result["title"]=result.title.str.encode('latin-1', 'ignore')
    result["title"]=result.title.str.decode('latin-1')
    result["text"]=result.text.str.encode('latin-1', 'ignore')
    result["text"]=result.text.str.decode('latin-1')
    result["summary"]=result.summary.str.encode('latin-1', 'ignore')
    result["summary"]=result.summary.str.decode('latin-1')

    html_text = ''
    for index, row in result.iterrows():
        html_text+=row[6]
    
    convert_html_to_pdf(html_text,'C:\\Users\\rolfl\\Downloads\\'+outFile+".pdf")